@@ -4,7 +4,7 @@
  *  this code, it is a sanity check rather deeply rooted in GiNaC's classes. */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2014 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,11 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include "exams.h"
+#include "ginac.h"
+using namespace GiNaC;
+
+#include <iostream>
+using namespace std;
 
 static unsigned exam_powerlaws1()
 {
@@ -65,14 +69,14 @@ static unsigned exam_powerlaws1()
 		return 1;
 	}
 	
-	ex e4 = e1.subs(lst(a==-1, b==2.5));
+	ex e4 = e1.subs(lst(a==-1, b==-2.5));
 	if (!(is_exactly_a<power>(e4) &&
 	      is_exactly_a<power>(e4.op(0)) &&
 	      is_exactly_a<symbol>(e4.op(0).op(0)) &&
 	      is_exactly_a<numeric>(e4.op(0).op(1)) &&
 	      is_exactly_a<numeric>(e4.op(1)) &&
-	      e4.is_equal(power(power(x,-1),2.5)) )) {
-		clog << "(x^a)^b, x symbolic, a==-1, b==2.5 wrong" << endl;
+	      e4.is_equal(power(power(x,-1),-2.5)) )) {
+		clog << "(x^a)^b, x symbolic, a==-1, b==-2.5 wrong" << endl;
 		clog << "returned: " << e4 << endl;
 		return 1;
 	}
@@ -289,7 +293,6 @@ unsigned exam_powerlaws()
 	unsigned result = 0;
 	
 	cout << "examining power laws" << flush;
-	clog << "----------power laws:" << endl;
 	
 	result += exam_powerlaws1();  cout << '.' << flush;
 	result += exam_powerlaws2();  cout << '.' << flush;
@@ -297,12 +300,10 @@ unsigned exam_powerlaws()
 	result += exam_powerlaws4();  cout << '.' << flush;
 	result += exam_powerlaws5();  cout << '.' << flush;
 	
-	if (!result) {
-		cout << " passed " << endl;
-		clog << "(no output)" << endl;
-	} else {
-		cout << " failed " << endl;
-	}
-	
 	return result;
 }
+
+int main(int argc, char** argv)
+{
+	return exam_powerlaws();
+}
