@@ -7,7 +7,7 @@
  *  after which e should be just a1^2. */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2014 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,7 +24,14 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include "times.h"
+#include "ginac.h"
+#include "timer.h"
+using namespace GiNaC;
+
+#include <iostream>
+#include <sstream>
+#include <vector>
+using namespace std;
 
 static unsigned expand_subs(unsigned size)
 {
@@ -58,16 +65,15 @@ unsigned time_dennyfliegner()
 	unsigned result = 0;
 	
 	cout << "timing commutative expansion and substitution" << flush;
-	clog << "-------commutative expansion and substitution:" << endl;
 	
 	vector<unsigned> sizes;
 	vector<double> times;
 	timer breitling;
 	
-	sizes.push_back(50);
 	sizes.push_back(100);
 	sizes.push_back(200);
 	sizes.push_back(400);
+	sizes.push_back(800);
 	
 	for (vector<unsigned>::iterator i=sizes.begin(); i!=sizes.end(); ++i) {
 		breitling.start();
@@ -76,20 +82,23 @@ unsigned time_dennyfliegner()
 		cout << '.' << flush;
 	}
 	
-	if (!result) {
-		cout << " passed ";
-		clog << "(no output)" << endl;
-	} else {
-		cout << " failed ";
-	}
 	// print the report:
 	cout << endl << "	size:  ";
 	for (vector<unsigned>::iterator i=sizes.begin(); i!=sizes.end(); ++i)
-		cout << '\t' << (*i);
+		cout << '\t' << *i;
 	cout << endl << "	time/s:";
 	for (vector<double>::iterator i=times.begin(); i!=times.end(); ++i)
-		cout << '\t' << int(1000*(*i))*0.001;
+		cout << '\t' << *i;
 	cout << endl;
 	
 	return result;
 }
+
+extern void randomify_symbol_serials();
+
+int main(int argc, char** argv)
+{
+	randomify_symbol_serials();
+	cout << setprecision(2) << showpoint;
+	return time_dennyfliegner();
+}
