@@ -3,7 +3,7 @@
  *  Definition of optimizing macros. */
 
 /*
- *  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2014 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,4 +31,9 @@
 #define likely(cond) (cond)
 #endif
 
+#ifdef _MSC_VER
+#define __func__ __FUNCTION__
+#define __alignof__ __alignof
+#endif
+
 #endif // ndef GINAC_COMPILER_DEP_H
