@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's light-weight expression handles. */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2014 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,20 +20,20 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <iostream>
-#include <stdexcept>
-
 #include "ex.h"
 #include "add.h"
 #include "mul.h"
 #include "ncmul.h"
 #include "numeric.h"
+#include "matrix.h"
 #include "power.h"
 #include "lst.h"
 #include "relational.h"
-#include "input_lexer.h"
 #include "utils.h"
 
+#include <iostream>
+#include <stdexcept>
+
 namespace GiNaC {
 
 //////////
@@ -94,7 +94,7 @@ ex ex::diff(const symbol & s, unsigned nth) const
 /** Check whether expression matches a specified pattern. */
 bool ex::match(const ex & pattern) const
 {
-	lst repl_lst;
+	exmap repl_lst;
 	return bp->match(pattern, repl_lst);
 }
 
@@ -102,12 +102,10 @@ bool ex::match(const ex & pattern) const
  *  the "found" list. If the expression itself matches the pattern, the
  *  children are not further examined. This function returns true when any
  *  matches were found. */
-bool ex::find(const ex & pattern, lst & found) const
+bool ex::find(const ex & pattern, exset& found) const
 {
 	if (match(pattern)) {
-		found.append(*this);
-		found.sort();
-		found.unique();
+		found.insert(*this);
 		return true;
 	}
 	bool any_found = false;
@@ -240,6 +238,31 @@ ex ex::rhs() const
 	return bp->op(1);
 }
 
+/** Check whether expression is a polynomial. */
+bool ex::is_polynomial(const ex & vars) const
+{
+	if (is_a<lst>(vars)) {
+		const lst & varlst = ex_to<lst>(vars);
+		for (lst::const_iterator i=varlst.begin(); i!=varlst.end(); ++i)
+			if (!bp->is_polynomial(*i))
+				return false;
+		return true;
+	}
+	else
+		return bp->is_polynomial(vars);
+}
+
+/** Check whether expression is zero or zero matrix. */
+bool ex::is_zero_matrix() const
+{
+	if (is_zero())
+		return  true;
+	else {
+		ex e = evalm();
+		return is_a<matrix>(e) && ex_to<matrix>(e).is_zero_matrix();
+	}
+}
+
 // private
 
 /** Make this ex writable (if more than one ex handle the same basic) by 
@@ -529,17 +552,6 @@ basic & ex::construct_from_double(double d)
 	return *bp;
 }
 
-ptr<basic> ex::construct_from_string_and_lst(const std::string &s, const ex &l)
-{
-	set_lexer_string(s);
-	set_lexer_symbols(l);
-	ginac_yyrestart(NULL);
-	if (ginac_yyparse())
-		throw (std::runtime_error(get_parser_error()));
-	else
-		return parsed_ex.bp;
-}
-	
 //////////
 // static member variables
 //////////
