@@ -6,7 +6,7 @@
  */
 
 /*
- *  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2014 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -113,12 +113,12 @@ public:
 			char* new_filename = new char[strlen(filename_pattern)+1];
 			strcpy(new_filename, filename_pattern);
 			if (!mktemp(new_filename)) {
-				delete new_filename;
+				delete[] new_filename;
 				throw std::runtime_error("mktemp failed");
 			}
 			filename = std::string(new_filename);
 			ofs.open(new_filename, std::ios::out);
-			delete new_filename;
+			delete[] new_filename;
 		} else {
 			// use parameter as filename
 			ofs.open(filename.c_str(), std::ios::out);
@@ -172,7 +172,7 @@ public:
 		for (std::vector<filedesc>::iterator it = filelist.begin(); it != filelist.end();) {
 			if (it->name == filename) {
 				clean_up(it);
-				filelist.erase(it);
+				it = filelist.erase(it);
 			} else {
 				++it;
 			}
