@@ -3,7 +3,7 @@
  *  Interface to GiNaC's indexed expressions. */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2014 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,16 +20,15 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef __GINAC_INDEXED_H__
-#define __GINAC_INDEXED_H__
-
-#include <map>
+#ifndef GINAC_INDEXED_H
+#define GINAC_INDEXED_H
 
 #include "exprseq.h"
 #include "wildcard.h"
 
-namespace GiNaC {
+#include <map>
 
+namespace GiNaC {
 
 class scalar_products;
 class symmetry;
@@ -147,13 +146,20 @@ public:
 	unsigned precedence() const {return 55;}
 	bool info(unsigned inf) const;
 	ex eval(int level = 0) const;
+	ex real_part() const;
+	ex imag_part() const;
 	exvector get_free_indices() const;
 
+	/** Save (a.k.a. serialize) indexed object into archive. */
+	void archive(archive_node& n) const;
+	/** Read (a.k.a. deserialize) indexed object from archive. */
+	void read_archive(const archive_node& n, lst& syms);
 protected:
 	ex derivative(const symbol & s) const;
 	ex thiscontainer(const exvector & v) const;
 	ex thiscontainer(std::auto_ptr<exvector> vp) const;
-	unsigned return_type() const { return return_types::commutative; }
+	unsigned return_type() const;
+	return_type_t return_type_tinfo() const { return op(0).return_type_tinfo(); }
 	ex expand(unsigned options = 0) const;
 
 	// new virtual functions which can be overridden by derived classes
@@ -195,6 +201,7 @@ protected:
 protected:
 	ex symtree; /**< Index symmetry (tree of symmetry objects) */
 };
+GINAC_DECLARE_UNARCHIVER(indexed);
 
 
 class spmapkey {
@@ -244,18 +251,27 @@ protected:
 
 // utility functions
 
-/** Specialization of is_exactly_a<indexed>(obj) for indexed objects. */
-template<> inline bool is_exactly_a<indexed>(const basic & obj)
-{
-	return obj.tinfo()==TINFO_indexed;
-}
-
 /** Returns all dummy indices from the expression */
 exvector get_all_dummy_indices(const ex & e);
 
+/** More reliable version of the form. The former assumes that e is an
+  * expanded epxression. */
+exvector get_all_dummy_indices_safely(const ex & e);
+
+/** Returns b with all dummy indices, which are listed in va, renamed 
+ *  if modify_va is set to TRUE all dummy indices of b will be appended to va */
+ex rename_dummy_indices_uniquely(exvector & va, const ex & b, bool modify_va = false);
+
 /** Returns b with all dummy indices, which are common with a, renamed */
 ex rename_dummy_indices_uniquely(const ex & a, const ex & b);
 
+/** Same as above, where va and vb contain the indices of a and b and are sorted */
+ex rename_dummy_indices_uniquely(const exvector & va, const exvector & vb, const ex & b);
+
+/** Similar to above, where va and vb are the same and the return value is a list of two lists 
+ *  for substitution in b */
+lst rename_dummy_indices_uniquely(const exvector & va, const exvector & vb);
+
 /** This function returns the given expression with expanded sums
  *  for all dummy index summations, where the dimensionality of 
  *  the dummy index is a nonnegative integer.
@@ -269,4 +285,4 @@ ex expand_dummy_sum(const ex & e, bool subs_idx = false);
 
 } // namespace GiNaC
 
-#endif // ndef __GINAC_INDEXED_H__
+#endif // ndef GINAC_INDEXED_H
