@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's initially known functions. */
 
 /*
- *  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2014 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -76,8 +76,46 @@ static ex conjugate_imag_part(const ex & arg)
 	return -arg.imag_part();
 }
 
+static bool func_arg_info(const ex & arg, unsigned inf)
+{
+	// for some functions we can return the info() of its argument
+	// (think of conjugate())
+	switch (inf) {
+		case info_flags::polynomial:
+		case info_flags::integer_polynomial:
+		case info_flags::cinteger_polynomial:
+		case info_flags::rational_polynomial:
+		case info_flags::real:
+		case info_flags::rational:
+		case info_flags::integer:
+		case info_flags::crational:
+		case info_flags::cinteger:
+		case info_flags::even:
+		case info_flags::odd:
+		case info_flags::prime:
+		case info_flags::crational_polynomial:
+		case info_flags::rational_function:
+		case info_flags::algebraic:
+		case info_flags::positive:
+		case info_flags::negative:
+		case info_flags::nonnegative:
+		case info_flags::posint:
+		case info_flags::negint:
+		case info_flags::nonnegint:
+		case info_flags::has_indices:
+			return arg.info(inf);
+	}
+	return false;
+}
+
+static bool conjugate_info(const ex & arg, unsigned inf)
+{
+	return func_arg_info(arg, inf);
+}
+
 REGISTER_FUNCTION(conjugate_function, eval_func(conjugate_eval).
                                       evalf_func(conjugate_evalf).
+                                      info_func(conjugate_info).
                                       print_func<print_latex>(conjugate_print_latex).
                                       conjugate_func(conjugate_conjugate).
                                       real_part_func(conjugate_real_part).
@@ -197,9 +235,46 @@ static ex abs_eval(const ex & arg)
 	if (is_ex_the_function(arg, abs))
 		return arg;
 
+	if (is_ex_the_function(arg, exp))
+		return exp(arg.op(0).real_part());
+
+	if (is_exactly_a<power>(arg)) {
+		const ex& base = arg.op(0);
+		const ex& exponent = arg.op(1);
+		if (base.info(info_flags::positive) || exponent.info(info_flags::real))
+			return pow(abs(base), exponent.real_part());
+	}
+
+	if (is_ex_the_function(arg, conjugate_function))
+		return abs(arg.op(0));
+
+	if (is_ex_the_function(arg, step))
+		return arg;
+
 	return abs(arg).hold();
 }
 
+static ex abs_expand(const ex & arg, unsigned options)
+{
+	if ((options & expand_options::expand_transcendental)
+		&& is_exactly_a<mul>(arg)) {
+		exvector prodseq;
+		prodseq.reserve(arg.nops());
+		for (const_iterator i = arg.begin(); i != arg.end(); ++i) {
+			if (options & expand_options::expand_function_args)
+				prodseq.push_back(abs(i->expand(options)));
+			else
+				prodseq.push_back(abs(*i));
+		}
+		return (new mul(prodseq))->setflag(status_flags::dynallocated | status_flags::expanded);
+	}
+
+	if (options & expand_options::expand_function_args)
+		return abs(arg.expand(options)).hold();
+	else
+		return abs(arg).hold();
+}
+
 static void abs_print_latex(const ex & arg, const print_context & c)
 {
 	c.s << "{|"; arg.print(c); c.s << "|}";
@@ -212,7 +287,7 @@ static void abs_print_csrc_float(const ex & arg, const print_context & c)
 
 static ex abs_conjugate(const ex & arg)
 {
-	return abs(arg);
+	return abs(arg).hold();
 }
 
 static ex abs_real_part(const ex & arg)
@@ -227,14 +302,44 @@ static ex abs_imag_part(const ex& arg)
 
 static ex abs_power(const ex & arg, const ex & exp)
 {
-	if (arg.is_equal(arg.conjugate()) && is_a<numeric>(exp) && ex_to<numeric>(exp).is_even())
+	if (arg.is_equal(arg.conjugate()) && ((is_a<numeric>(exp) && ex_to<numeric>(exp).is_even())
+						|| exp.info(info_flags::even)))
 		return power(arg, exp);
 	else
 		return power(abs(arg), exp).hold();
 }
 
+bool abs_info(const ex & arg, unsigned inf)
+{
+	switch (inf) {
+		case info_flags::integer:
+		case info_flags::even:
+		case info_flags::odd:
+		case info_flags::prime:
+			return arg.info(inf);
+		case info_flags::nonnegint:
+			return arg.info(info_flags::integer);
+		case info_flags::nonnegative:
+		case info_flags::real:
+			return true;
+		case info_flags::negative:
+			return false;
+		case info_flags::positive:
+			return arg.info(info_flags::positive) || arg.info(info_flags::negative);
+		case info_flags::has_indices: {
+			if (arg.info(info_flags::has_indices))
+				return true;
+			else
+				return false;
+		}
+	}
+	return false;
+}
+
 REGISTER_FUNCTION(abs, eval_func(abs_eval).
                        evalf_func(abs_evalf).
+                       expand_func(abs_expand).
+                       info_func(abs_info).
                        print_func<print_latex>(abs_print_latex).
                        print_func<print_csrc_float>(abs_print_csrc_float).
                        print_func<print_csrc_double>(abs_print_csrc_float).
@@ -319,8 +424,8 @@ REGISTER_FUNCTION(step, eval_func(step_eval).
                         evalf_func(step_evalf).
                         series_func(step_series).
                         conjugate_func(step_conjugate).
-								real_part_func(step_real_part).
-								imag_part_func(step_imag_part));
+                        real_part_func(step_real_part).
+                        imag_part_func(step_imag_part));
 
 //////////
 // Complex sign
@@ -398,7 +503,7 @@ static ex csgn_power(const ex & arg, const ex & exp)
 {
 	if (is_a<numeric>(exp) && exp.info(info_flags::positive) && ex_to<numeric>(exp).is_integer()) {
 		if (ex_to<numeric>(exp).is_odd())
-			return csgn(arg);
+			return csgn(arg).hold();
 		else
 			return power(csgn(arg), _ex2).hold();
 	} else
@@ -489,7 +594,7 @@ static ex eta_series(const ex & x, const ex & y,
 
 static ex eta_conjugate(const ex & x, const ex & y)
 {
-	return -eta(x, y);
+	return -eta(x, y).hold();
 }
 
 static ex eta_real_part(const ex & x, const ex & y)
@@ -643,11 +748,26 @@ static ex Li2_series(const ex &x, const relational &rel, int order, unsigned opt
 	throw do_taylor();  // caught by function::series()
 }
 
+static ex Li2_conjugate(const ex & x)
+{
+	// conjugate(Li2(x))==Li2(conjugate(x)) unless on the branch cuts which
+	// run along the positive real axis beginning at 1.
+	if (x.info(info_flags::negative)) {
+		return Li2(x).hold();
+	}
+	if (is_exactly_a<numeric>(x) &&
+	    (!x.imag_part().is_zero() || x < *_num1_p)) {
+		return Li2(x.conjugate());
+	}
+	return conjugate_function(Li2(x)).hold();
+}
+
 REGISTER_FUNCTION(Li2, eval_func(Li2_eval).
                        evalf_func(Li2_evalf).
                        derivative_func(Li2_deriv).
                        series_func(Li2_series).
-                       latex_name("\\mbox{Li}_2"));
+                       conjugate_func(Li2_conjugate).
+                       latex_name("\\mathrm{Li}_2"));
 
 //////////
 // trilogarithm
@@ -661,7 +781,7 @@ static ex Li3_eval(const ex & x)
 }
 
 REGISTER_FUNCTION(Li3, eval_func(Li3_eval).
-                       latex_name("\\mbox{Li}_3"));
+                       latex_name("\\mathrm{Li}_3"));
 
 //////////
 // Derivatives of Riemann's Zeta-function  zetaderiv(0,x)==zeta(x)
@@ -672,7 +792,7 @@ static ex zetaderiv_eval(const ex & n, const ex & x)
 	if (n.info(info_flags::numeric)) {
 		// zetaderiv(0,x) -> zeta(x)
 		if (n.is_zero())
-			return zeta(x);
+			return zeta(x).hold();
 	}
 	
 	return zetaderiv(n, x).hold();
@@ -992,9 +1112,24 @@ fsolve(const ex& f_in, const symbol& x, const numeric& x1, const numeric& x2)
 	do {
 		xxprev = xx[side];
 		fxprev = fx[side];
-		xx[side] += ex_to<numeric>(ff.subs(x==xx[side]).evalf());
-		fx[side] = ex_to<numeric>(f.subs(x==xx[side]).evalf());
-		if ((side==0 && xx[0]<xxprev) || (side==1 && xx[1]>xxprev) || xx[0]>xx[1]) {
+		ex dx_ = ff.subs(x == xx[side]).evalf();
+		if (!is_a<numeric>(dx_))
+			throw std::runtime_error("fsolve(): function derivative does not evaluate numerically");
+		xx[side] += ex_to<numeric>(dx_);
+		// Now check if Newton-Raphson method shot out of the interval 
+		bool bad_shot = (side == 0 && xx[0] < xxprev) || 
+				(side == 1 && xx[1] > xxprev) || xx[0] > xx[1];
+		if (!bad_shot) {
+			// Compute f(x) only if new x is inside the interval.
+			// The function might be difficult to compute numerically
+			// or even ill defined outside the interval. Also it's
+			// a small optimization. 
+			ex f_x = f.subs(x == xx[side]).evalf();
+			if (!is_a<numeric>(f_x))
+				throw std::runtime_error("fsolve(): function does not evaluate numerically");
+			fx[side] = ex_to<numeric>(f_x);
+		}
+		if (bad_shot) {
 			// Oops, Newton-Raphson method shot out of the interval.
 			// Restore, and try again with the other side instead!
 			xx[side] = xxprev;
@@ -1002,8 +1137,16 @@ fsolve(const ex& f_in, const symbol& x, const numeric& x1, const numeric& x2)
 			side = !side;
 			xxprev = xx[side];
 			fxprev = fx[side];
-			xx[side] += ex_to<numeric>(ff.subs(x==xx[side]).evalf());
-			fx[side] = ex_to<numeric>(f.subs(x==xx[side]).evalf());
+
+			ex dx_ = ff.subs(x == xx[side]).evalf();
+			if (!is_a<numeric>(dx_))
+				throw std::runtime_error("fsolve(): function derivative does not evaluate numerically [2]");
+			xx[side] += ex_to<numeric>(dx_);
+
+			ex f_x = f.subs(x==xx[side]).evalf();
+			if (!is_a<numeric>(f_x))
+				throw std::runtime_error("fsolve(): function does not evaluate numerically [2]");
+			fx[side] = ex_to<numeric>(f_x);
 		}
 		if ((fx[side]<0 && fx[!side]<0) || (fx[side]>0 && fx[!side]>0)) {
 			// Oops, the root isn't bracketed any more.
@@ -1027,7 +1170,10 @@ fsolve(const ex& f_in, const symbol& x, const numeric& x1, const numeric& x2)
 			static const double secant_weight = 0.984375;  // == 63/64 < 1
 			numeric xxmid = (1-secant_weight)*0.5*(xx[0]+xx[1])
 			    + secant_weight*(xx[0]+fx[0]*(xx[0]-xx[1])/(fx[1]-fx[0]));
-			numeric fxmid = ex_to<numeric>(f.subs(x==xxmid).evalf());
+			ex fxmid_ = f.subs(x == xxmid).evalf();
+			if (!is_a<numeric>(fxmid_))
+				throw std::runtime_error("fsolve(): function does not evaluate numerically [3]");
+			numeric fxmid = ex_to<numeric>(fxmid_);
 			if (fxmid.is_zero()) {
 				// Luck strikes...
 				return xxmid;
