@@ -4,7 +4,7 @@
  *  functions. */
 
 /*
- *  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2014 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,6 +24,8 @@
 #include "inifcns.h"
 #include "ex.h"
 #include "constant.h"
+#include "add.h"
+#include "mul.h"
 #include "numeric.h"
 #include "power.h"
 #include "operators.h"
@@ -81,6 +83,27 @@ static ex exp_eval(const ex & x)
 	return exp(x).hold();
 }
 
+static ex exp_expand(const ex & arg, unsigned options)
+{
+	ex exp_arg;
+	if (options & expand_options::expand_function_args)
+		exp_arg = arg.expand(options);
+	else
+		exp_arg=arg;
+
+	if ((options & expand_options::expand_transcendental)
+		&& is_exactly_a<add>(exp_arg)) {
+		exvector prodseq;
+		prodseq.reserve(exp_arg.nops());
+		for (const_iterator i = exp_arg.begin(); i != exp_arg.end(); ++i)
+			prodseq.push_back(exp(*i));
+
+		return (new mul(prodseq))->setflag(status_flags::dynallocated | status_flags::expanded);
+	}
+
+	return exp(exp_arg).hold();
+}
+
 static ex exp_deriv(const ex & x, unsigned deriv_param)
 {
 	GINAC_ASSERT(deriv_param==0);
@@ -99,11 +122,19 @@ static ex exp_imag_part(const ex & x)
 	return exp(GiNaC::real_part(x))*sin(GiNaC::imag_part(x));
 }
 
+static ex exp_conjugate(const ex & x)
+{
+	// conjugate(exp(x))==exp(conjugate(x))
+	return exp(x.conjugate());
+}
+
 REGISTER_FUNCTION(exp, eval_func(exp_eval).
                        evalf_func(exp_evalf).
+                       expand_func(exp_expand).
                        derivative_func(exp_deriv).
                        real_part_func(exp_real_part).
                        imag_part_func(exp_imag_part).
+                       conjugate_func(exp_conjugate).
                        latex_name("\\exp"));
 
 //////////
@@ -143,7 +174,7 @@ static ex log_eval(const ex & x)
 		if (t.info(info_flags::real))
 			return t;
 	}
-	
+
 	return log(x).hold();
 }
 
@@ -173,6 +204,10 @@ static ex log_series(const ex &arg,
 	if (arg_pt.is_zero())
 		must_expand_arg = true;
 	
+	if (arg.diff(ex_to<symbol>(rel.lhs())).is_zero()) {
+		throw do_taylor();
+	}
+
 	if (must_expand_arg) {
 		// method:
 		// This is the branch point: Series expand the argument first, then
@@ -258,12 +293,75 @@ static ex log_imag_part(const ex & x)
 	return atan2(GiNaC::imag_part(x), GiNaC::real_part(x));
 }
 
+static ex log_expand(const ex & arg, unsigned options)
+{
+	if ((options & expand_options::expand_transcendental)
+		&& is_exactly_a<mul>(arg) && !arg.info(info_flags::indefinite)) {
+		exvector sumseq;
+		exvector prodseq;
+		sumseq.reserve(arg.nops());
+		prodseq.reserve(arg.nops());
+		bool possign=true;
+
+		// searching for positive/negative factors
+		for (const_iterator i = arg.begin(); i != arg.end(); ++i) {
+			ex e;
+			if (options & expand_options::expand_function_args)
+				e=i->expand(options);
+			else
+				e=*i;
+			if (e.info(info_flags::positive))
+				sumseq.push_back(log(e));
+			else if (e.info(info_flags::negative)) {
+				sumseq.push_back(log(-e));
+				possign = !possign;
+			} else
+				prodseq.push_back(e);
+		}
+
+		if (sumseq.size() > 0) {
+			ex newarg;
+			if (options & expand_options::expand_function_args)
+				newarg=((possign?_ex1:_ex_1)*mul(prodseq)).expand(options);
+			else {
+				newarg=(possign?_ex1:_ex_1)*mul(prodseq);
+				ex_to<basic>(newarg).setflag(status_flags::purely_indefinite);
+			}
+			return add(sumseq)+log(newarg);
+		} else {
+			if (!(options & expand_options::expand_function_args))
+				ex_to<basic>(arg).setflag(status_flags::purely_indefinite);
+		}
+	}
+
+	if (options & expand_options::expand_function_args)
+		return log(arg.expand(options)).hold();
+	else
+		return log(arg).hold();
+}
+
+static ex log_conjugate(const ex & x)
+{
+	// conjugate(log(x))==log(conjugate(x)) unless on the branch cut which
+	// runs along the negative real axis.
+	if (x.info(info_flags::positive)) {
+		return log(x);
+	}
+	if (is_exactly_a<numeric>(x) &&
+	    !x.imag_part().is_zero()) {
+		return log(x.conjugate());
+	}
+	return conjugate_function(log(x)).hold();
+}
+
 REGISTER_FUNCTION(log, eval_func(log_eval).
                        evalf_func(log_evalf).
+                       expand_func(log_expand).
                        derivative_func(log_deriv).
                        series_func(log_series).
                        real_part_func(log_real_part).
                        imag_part_func(log_imag_part).
+                       conjugate_func(log_conjugate).
                        latex_name("\\ln"));
 
 //////////
@@ -359,11 +457,18 @@ static ex sin_imag_part(const ex & x)
 	return sinh(GiNaC::imag_part(x))*cos(GiNaC::real_part(x));
 }
 
+static ex sin_conjugate(const ex & x)
+{
+	// conjugate(sin(x))==sin(conjugate(x))
+	return sin(x.conjugate());
+}
+
 REGISTER_FUNCTION(sin, eval_func(sin_eval).
                        evalf_func(sin_evalf).
                        derivative_func(sin_deriv).
                        real_part_func(sin_real_part).
                        imag_part_func(sin_imag_part).
+                       conjugate_func(sin_conjugate).
                        latex_name("\\sin"));
 
 //////////
@@ -459,11 +564,18 @@ static ex cos_imag_part(const ex & x)
 	return -sinh(GiNaC::imag_part(x))*sin(GiNaC::real_part(x));
 }
 
+static ex cos_conjugate(const ex & x)
+{
+	// conjugate(cos(x))==cos(conjugate(x))
+	return cos(x.conjugate());
+}
+
 REGISTER_FUNCTION(cos, eval_func(cos_eval).
                        evalf_func(cos_evalf).
                        derivative_func(cos_deriv).
                        real_part_func(cos_real_part).
                        imag_part_func(cos_imag_part).
+                       conjugate_func(cos_conjugate).
                        latex_name("\\cos"));
 
 //////////
@@ -576,12 +688,19 @@ static ex tan_series(const ex &x,
 	return (sin(x)/cos(x)).series(rel, order, options);
 }
 
+static ex tan_conjugate(const ex & x)
+{
+	// conjugate(tan(x))==tan(conjugate(x))
+	return tan(x.conjugate());
+}
+
 REGISTER_FUNCTION(tan, eval_func(tan_eval).
                        evalf_func(tan_evalf).
                        derivative_func(tan_deriv).
                        series_func(tan_series).
                        real_part_func(tan_real_part).
                        imag_part_func(tan_imag_part).
+                       conjugate_func(tan_conjugate).
                        latex_name("\\tan"));
 
 //////////
@@ -640,9 +759,21 @@ static ex asin_deriv(const ex & x, unsigned deriv_param)
 	return power(1-power(x,_ex2),_ex_1_2);
 }
 
+static ex asin_conjugate(const ex & x)
+{
+	// conjugate(asin(x))==asin(conjugate(x)) unless on the branch cuts which
+	// run along the real axis outside the interval [-1, +1].
+	if (is_exactly_a<numeric>(x) &&
+	    (!x.imag_part().is_zero() || (x > *_num_1_p && x < *_num1_p))) {
+		return asin(x.conjugate());
+	}
+	return conjugate_function(asin(x)).hold();
+}
+
 REGISTER_FUNCTION(asin, eval_func(asin_eval).
                         evalf_func(asin_evalf).
                         derivative_func(asin_deriv).
+                        conjugate_func(asin_conjugate).
                         latex_name("\\arcsin"));
 
 //////////
@@ -701,9 +832,21 @@ static ex acos_deriv(const ex & x, unsigned deriv_param)
 	return -power(1-power(x,_ex2),_ex_1_2);
 }
 
+static ex acos_conjugate(const ex & x)
+{
+	// conjugate(acos(x))==acos(conjugate(x)) unless on the branch cuts which
+	// run along the real axis outside the interval [-1, +1].
+	if (is_exactly_a<numeric>(x) &&
+	    (!x.imag_part().is_zero() || (x > *_num_1_p && x < *_num1_p))) {
+		return acos(x.conjugate());
+	}
+	return conjugate_function(acos(x)).hold();
+}
+
 REGISTER_FUNCTION(acos, eval_func(acos_eval).
                         evalf_func(acos_evalf).
                         derivative_func(acos_deriv).
+                        conjugate_func(acos_conjugate).
                         latex_name("\\arccos"));
 
 //////////
@@ -800,10 +943,27 @@ static ex atan_series(const ex &arg,
 	throw do_taylor();
 }
 
+static ex atan_conjugate(const ex & x)
+{
+	// conjugate(atan(x))==atan(conjugate(x)) unless on the branch cuts which
+	// run along the imaginary axis outside the interval [-I, +I].
+	if (x.info(info_flags::real))
+		return atan(x);
+	if (is_exactly_a<numeric>(x)) {
+		const numeric x_re = ex_to<numeric>(x.real_part());
+		const numeric x_im = ex_to<numeric>(x.imag_part());
+		if (!x_re.is_zero() ||
+		    (x_im > *_num_1_p && x_im < *_num1_p))
+			return atan(x.conjugate());
+	}
+	return conjugate_function(atan(x)).hold();
+}
+
 REGISTER_FUNCTION(atan, eval_func(atan_eval).
                         evalf_func(atan_evalf).
                         derivative_func(atan_deriv).
                         series_func(atan_series).
+                        conjugate_func(atan_conjugate).
                         latex_name("\\arctan"));
 
 //////////
@@ -822,65 +982,68 @@ static ex atan2_eval(const ex & y, const ex & x)
 {
 	if (y.is_zero()) {
 
-		// atan(0, 0) -> 0
+		// atan2(0, 0) -> 0
 		if (x.is_zero())
 			return _ex0;
 
-		// atan(0, x), x real and positive -> 0
+		// atan2(0, x), x real and positive -> 0
 		if (x.info(info_flags::positive))
 			return _ex0;
 
-		// atan(0, x), x real and negative -> Pi
+		// atan2(0, x), x real and negative -> Pi
 		if (x.info(info_flags::negative))
 			return Pi;
 	}
 
 	if (x.is_zero()) {
 
-		// atan(y, 0), y real and positive -> Pi/2
+		// atan2(y, 0), y real and positive -> Pi/2
 		if (y.info(info_flags::positive))
 			return _ex1_2*Pi;
 
-		// atan(y, 0), y real and negative -> -Pi/2
+		// atan2(y, 0), y real and negative -> -Pi/2
 		if (y.info(info_flags::negative))
 			return _ex_1_2*Pi;
 	}
 
 	if (y.is_equal(x)) {
 
-		// atan(y, y), y real and positive -> Pi/4
+		// atan2(y, y), y real and positive -> Pi/4
 		if (y.info(info_flags::positive))
 			return _ex1_4*Pi;
 
-		// atan(y, y), y real and negative -> -3/4*Pi
+		// atan2(y, y), y real and negative -> -3/4*Pi
 		if (y.info(info_flags::negative))
 			return numeric(-3, 4)*Pi;
 	}
 
 	if (y.is_equal(-x)) {
 
-		// atan(y, -y), y real and positive -> 3*Pi/4
+		// atan2(y, -y), y real and positive -> 3*Pi/4
 		if (y.info(info_flags::positive))
 			return numeric(3, 4)*Pi;
 
-		// atan(y, -y), y real and negative -> -Pi/4
+		// atan2(y, -y), y real and negative -> -Pi/4
 		if (y.info(info_flags::negative))
 			return _ex_1_4*Pi;
 	}
 
-	// atan(float, float) -> float
+	// atan2(float, float) -> float
 	if (is_a<numeric>(y) && !y.info(info_flags::crational) &&
 	    is_a<numeric>(x) && !x.info(info_flags::crational))
 		return atan(ex_to<numeric>(y), ex_to<numeric>(x));
 
-	// atan(real, real) -> atan(y/x) +/- Pi
+	// atan2(real, real) -> atan(y/x) +/- Pi
 	if (y.info(info_flags::real) && x.info(info_flags::real)) {
 		if (x.info(info_flags::positive))
 			return atan(y/x);
-		else if (y.info(info_flags::positive))
-			return atan(y/x)+Pi;
-		else
-			return atan(y/x)-Pi;
+
+		if (x.info(info_flags::negative)) {
+			if (y.info(info_flags::positive))
+				return atan(y/x)+Pi;
+			if (y.info(info_flags::negative))
+				return atan(y/x)-Pi;
+		}
 	}
 
 	return atan2(y, x).hold();
@@ -891,10 +1054,10 @@ static ex atan2_deriv(const ex & y, const ex & x, unsigned deriv_param)
 	GINAC_ASSERT(deriv_param<2);
 	
 	if (deriv_param==0) {
-		// d/dy atan(y,x)
+		// d/dy atan2(y,x)
 		return x*power(power(x,_ex2)+power(y,_ex2),_ex_1);
 	}
-	// d/dx atan(y,x)
+	// d/dx atan2(y,x)
 	return -y*power(power(x,_ex2)+power(y,_ex2),_ex_1);
 }
 
@@ -972,11 +1135,18 @@ static ex sinh_imag_part(const ex & x)
 	return cosh(GiNaC::real_part(x))*sin(GiNaC::imag_part(x));
 }
 
+static ex sinh_conjugate(const ex & x)
+{
+	// conjugate(sinh(x))==sinh(conjugate(x))
+	return sinh(x.conjugate());
+}
+
 REGISTER_FUNCTION(sinh, eval_func(sinh_eval).
                         evalf_func(sinh_evalf).
                         derivative_func(sinh_deriv).
                         real_part_func(sinh_real_part).
                         imag_part_func(sinh_imag_part).
+                        conjugate_func(sinh_conjugate).
                         latex_name("\\sinh"));
 
 //////////
@@ -1049,11 +1219,18 @@ static ex cosh_imag_part(const ex & x)
 	return sinh(GiNaC::real_part(x))*sin(GiNaC::imag_part(x));
 }
 
+static ex cosh_conjugate(const ex & x)
+{
+	// conjugate(cosh(x))==cosh(conjugate(x))
+	return cosh(x.conjugate());
+}
+
 REGISTER_FUNCTION(cosh, eval_func(cosh_eval).
                         evalf_func(cosh_evalf).
                         derivative_func(cosh_deriv).
                         real_part_func(cosh_real_part).
                         imag_part_func(cosh_imag_part).
+                        conjugate_func(cosh_conjugate).
                         latex_name("\\cosh"));
 
 //////////
@@ -1146,12 +1323,19 @@ static ex tanh_imag_part(const ex & x)
 	return tan(b)/(1+power(tanh(a),2)*power(tan(b),2));
 }
 
+static ex tanh_conjugate(const ex & x)
+{
+	// conjugate(tanh(x))==tanh(conjugate(x))
+	return tanh(x.conjugate());
+}
+
 REGISTER_FUNCTION(tanh, eval_func(tanh_eval).
                         evalf_func(tanh_evalf).
                         derivative_func(tanh_deriv).
                         series_func(tanh_series).
                         real_part_func(tanh_real_part).
                         imag_part_func(tanh_imag_part).
+                        conjugate_func(tanh_conjugate).
                         latex_name("\\tanh"));
 
 //////////
@@ -1194,9 +1378,26 @@ static ex asinh_deriv(const ex & x, unsigned deriv_param)
 	return power(_ex1+power(x,_ex2),_ex_1_2);
 }
 
+static ex asinh_conjugate(const ex & x)
+{
+	// conjugate(asinh(x))==asinh(conjugate(x)) unless on the branch cuts which
+	// run along the imaginary axis outside the interval [-I, +I].
+	if (x.info(info_flags::real))
+		return asinh(x);
+	if (is_exactly_a<numeric>(x)) {
+		const numeric x_re = ex_to<numeric>(x.real_part());
+		const numeric x_im = ex_to<numeric>(x.imag_part());
+		if (!x_re.is_zero() ||
+		    (x_im > *_num_1_p && x_im < *_num1_p))
+			return asinh(x.conjugate());
+	}
+	return conjugate_function(asinh(x)).hold();
+}
+
 REGISTER_FUNCTION(asinh, eval_func(asinh_eval).
                          evalf_func(asinh_evalf).
-                         derivative_func(asinh_deriv));
+                         derivative_func(asinh_deriv).
+                         conjugate_func(asinh_conjugate));
 
 //////////
 // inverse hyperbolic cosine (trigonometric function)
@@ -1246,9 +1447,21 @@ static ex acosh_deriv(const ex & x, unsigned deriv_param)
 	return power(x+_ex_1,_ex_1_2)*power(x+_ex1,_ex_1_2);
 }
 
+static ex acosh_conjugate(const ex & x)
+{
+	// conjugate(acosh(x))==acosh(conjugate(x)) unless on the branch cut
+	// which runs along the real axis from +1 to -inf.
+	if (is_exactly_a<numeric>(x) &&
+	    (!x.imag_part().is_zero() || x > *_num1_p)) {
+		return acosh(x.conjugate());
+	}
+	return conjugate_function(acosh(x)).hold();
+}
+
 REGISTER_FUNCTION(acosh, eval_func(acosh_eval).
                          evalf_func(acosh_evalf).
-                         derivative_func(acosh_deriv));
+                         derivative_func(acosh_deriv).
+                         conjugate_func(acosh_conjugate));
 
 //////////
 // inverse hyperbolic tangent (trigonometric function)
@@ -1337,10 +1550,22 @@ static ex atanh_series(const ex &arg,
 	throw do_taylor();
 }
 
+static ex atanh_conjugate(const ex & x)
+{
+	// conjugate(atanh(x))==atanh(conjugate(x)) unless on the branch cuts which
+	// run along the real axis outside the interval [-1, +1].
+	if (is_exactly_a<numeric>(x) &&
+	    (!x.imag_part().is_zero() || (x > *_num_1_p && x < *_num1_p))) {
+		return atanh(x.conjugate());
+	}
+	return conjugate_function(atanh(x)).hold();
+}
+
 REGISTER_FUNCTION(atanh, eval_func(atanh_eval).
                          evalf_func(atanh_evalf).
                          derivative_func(atanh_deriv).
-                         series_func(atanh_series));
+                         series_func(atanh_series).
+                         conjugate_func(atanh_conjugate));
 
 
 } // namespace GiNaC
