@@ -3,7 +3,7 @@
  *  Implementation of symbolic matrices */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2014 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,13 +20,6 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <string>
-#include <iostream>
-#include <sstream>
-#include <algorithm>
-#include <map>
-#include <stdexcept>
-
 #include "matrix.h"
 #include "numeric.h"
 #include "lst.h"
@@ -40,6 +33,13 @@
 #include "archive.h"
 #include "utils.h"
 
+#include <algorithm>
+#include <iostream>
+#include <map>
+#include <sstream>
+#include <stdexcept>
+#include <string>
+
 namespace GiNaC {
 
 GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(matrix, basic,
@@ -53,7 +53,7 @@ GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(matrix, basic,
 //////////
 
 /** Default ctor.  Initializes to 1 x 1-dimensional zero-matrix. */
-matrix::matrix() : inherited(&matrix::tinfo_static), row(1), col(1), m(1, _ex0)
+matrix::matrix() : row(1), col(1), m(1, _ex0)
 {
 	setflag(status_flags::not_shareable);
 }
@@ -68,8 +68,7 @@ matrix::matrix() : inherited(&matrix::tinfo_static), row(1), col(1), m(1, _ex0)
  *
  *  @param r number of rows
  *  @param c number of cols */
-matrix::matrix(unsigned r, unsigned c)
-  : inherited(&matrix::tinfo_static), row(r), col(c), m(r*c, _ex0)
+matrix::matrix(unsigned r, unsigned c) : row(r), col(c), m(r*c, _ex0)
 {
 	setflag(status_flags::not_shareable);
 }
@@ -78,7 +77,7 @@ matrix::matrix(unsigned r, unsigned c)
 
 /** Ctor from representation, for internal use only. */
 matrix::matrix(unsigned r, unsigned c, const exvector & m2)
-  : inherited(&matrix::tinfo_static), row(r), col(c), m(m2)
+  : row(r), col(c), m(m2)
 {
 	setflag(status_flags::not_shareable);
 }
@@ -88,7 +87,7 @@ matrix::matrix(unsigned r, unsigned c, const exvector & m2)
  *  If the list has more elements than the matrix, the excessive elements are
  *  thrown away. */
 matrix::matrix(unsigned r, unsigned c, const lst & l)
-  : inherited(&matrix::tinfo_static), row(r), col(c), m(r*c, _ex0)
+  : row(r), col(c), m(r*c, _ex0)
 {
 	setflag(status_flags::not_shareable);
 
@@ -106,22 +105,25 @@ matrix::matrix(unsigned r, unsigned c, const lst & l)
 // archiving
 //////////
 
-matrix::matrix(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst)
+void matrix::read_archive(const archive_node &n, lst &sym_lst)
 {
-	setflag(status_flags::not_shareable);
+	inherited::read_archive(n, sym_lst);
 
 	if (!(n.find_unsigned("row", row)) || !(n.find_unsigned("col", col)))
 		throw (std::runtime_error("unknown matrix dimensions in archive"));
 	m.reserve(row * col);
+	// XXX: default ctor inserts a zero element, we need to erase it here.
+	m.pop_back();
 	archive_node::archive_node_cit first = n.find_first("m");
 	archive_node::archive_node_cit last = n.find_last("m");
 	++last;
-	for (archive_node::archive_node_cit i=first; i<last; ++i) {
+	for (archive_node::archive_node_cit i=first; i != last; ++i) {
 		ex e;
 		n.find_ex_by_loc(i, e, sym_lst);
 		m.push_back(e);
 	}
 }
+GINAC_BIND_UNARCHIVER(matrix);
 
 void matrix::archive(archive_node &n) const
 {
@@ -135,8 +137,6 @@ void matrix::archive(archive_node &n) const
 	}
 }
 
-DEFAULT_UNARCHIVE(matrix)
-
 //////////
 // functions overriding virtual functions from base classes
 //////////
@@ -222,7 +222,7 @@ ex matrix::eval(int level) const
 			m2[r*col+c] = m[r*col+c].eval(level);
 	
 	return (new matrix(row, col, m2))->setflag(status_flags::dynallocated |
-											   status_flags::evaluated);
+	                                           status_flags::evaluated);
 }
 
 ex matrix::subs(const exmap & mp, unsigned options) const
@@ -1414,18 +1414,27 @@ int matrix::fraction_free_elimination(const bool det)
 	
 	unsigned r0 = 0;
 	for (unsigned c0=0; c0<n && r0<m-1; ++c0) {
-		int indx = tmp_n.pivot(r0, c0, true);
-		if (indx==-1) {
+		// When trying to find a pivot, we should try a bit harder than expand().
+		// Searching the first non-zero element in-place here instead of calling
+		// pivot() allows us to do no more substitutions and back-substitutions
+		// than are actually necessary.
+		unsigned indx = r0;
+		while ((indx<m) &&
+		       (tmp_n[indx*n+c0].subs(srl, subs_options::no_pattern).expand().is_zero()))
+			++indx;
+		if (indx==m) {
+			// all elements in column c0 below row r0 vanish
 			sign = 0;
 			if (det)
 				return 0;
-		}
-		if (indx>=0) {
-			if (indx>0) {
+		} else {
+			if (indx>r0) {
+				// Matrix needs pivoting, swap rows r0 and indx of tmp_n and tmp_d.
 				sign = -sign;
-				// tmp_n's rows r0 and indx were swapped, do the same in tmp_d:
-				for (unsigned c=c0; c<n; ++c)
+				for (unsigned c=c0; c<n; ++c) {
+					tmp_n.m[n*indx+c].swap(tmp_n.m[n*r0+c]);
 					tmp_d.m[n*indx+c].swap(tmp_d.m[n*r0+c]);
+				}
 			}
 			for (unsigned r2=r0+1; r2<m; ++r2) {
 				for (unsigned c=c0+1; c<n; ++c) {
