@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's products of expressions. */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2014 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,11 +20,6 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <iostream>
-#include <vector>
-#include <stdexcept>
-#include <limits>
-
 #include "mul.h"
 #include "add.h"
 #include "power.h"
@@ -37,6 +32,11 @@
 #include "symbol.h"
 #include "compiler.h"
 
+#include <iostream>
+#include <limits>
+#include <stdexcept>
+#include <vector>
+
 namespace GiNaC {
 
 GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(mul, expairseq,
@@ -53,7 +53,6 @@ GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(mul, expairseq,
 
 mul::mul()
 {
-	tinfo_key = &mul::tinfo_static;
 }
 
 //////////
@@ -64,7 +63,6 @@ mul::mul()
 
 mul::mul(const ex & lh, const ex & rh)
 {
-	tinfo_key = &mul::tinfo_static;
 	overall_coeff = _ex1;
 	construct_from_2_ex(lh,rh);
 	GINAC_ASSERT(is_canonical());
@@ -72,7 +70,6 @@ mul::mul(const ex & lh, const ex & rh)
 
 mul::mul(const exvector & v)
 {
-	tinfo_key = &mul::tinfo_static;
 	overall_coeff = _ex1;
 	construct_from_exvector(v);
 	GINAC_ASSERT(is_canonical());
@@ -80,7 +77,6 @@ mul::mul(const exvector & v)
 
 mul::mul(const epvector & v)
 {
-	tinfo_key = &mul::tinfo_static;
 	overall_coeff = _ex1;
 	construct_from_epvector(v);
 	GINAC_ASSERT(is_canonical());
@@ -88,7 +84,6 @@ mul::mul(const epvector & v)
 
 mul::mul(const epvector & v, const ex & oc, bool do_index_renaming)
 {
-	tinfo_key = &mul::tinfo_static;
 	overall_coeff = oc;
 	construct_from_epvector(v, do_index_renaming);
 	GINAC_ASSERT(is_canonical());
@@ -96,7 +91,6 @@ mul::mul(const epvector & v, const ex & oc, bool do_index_renaming)
 
 mul::mul(std::auto_ptr<epvector> vp, const ex & oc, bool do_index_renaming)
 {
-	tinfo_key = &mul::tinfo_static;
 	GINAC_ASSERT(vp.get()!=0);
 	overall_coeff = oc;
 	construct_from_epvector(*vp, do_index_renaming);
@@ -105,7 +99,6 @@ mul::mul(std::auto_ptr<epvector> vp, const ex & oc, bool do_index_renaming)
 
 mul::mul(const ex & lh, const ex & mh, const ex & rh)
 {
-	tinfo_key = &mul::tinfo_static;
 	exvector factors;
 	factors.reserve(3);
 	factors.push_back(lh);
@@ -120,8 +113,6 @@ mul::mul(const ex & lh, const ex & mh, const ex & rh)
 // archiving
 //////////
 
-DEFAULT_ARCHIVING(mul)
-
 //////////
 // functions overriding virtual functions from base classes
 //////////
@@ -287,6 +278,12 @@ bool mul::info(unsigned inf) const
 		case info_flags::integer_polynomial:
 		case info_flags::cinteger_polynomial:
 		case info_flags::rational_polynomial:
+		case info_flags::real:
+		case info_flags::rational:
+		case info_flags::integer:
+		case info_flags::crational:
+		case info_flags::cinteger:
+		case info_flags::even:
 		case info_flags::crational_polynomial:
 		case info_flags::rational_function: {
 			epvector::const_iterator i = seq.begin(), end = seq.end();
@@ -295,6 +292,8 @@ bool mul::info(unsigned inf) const
 					return false;
 				++i;
 			}
+			if (overall_coeff.is_equal(*_num1_p) && inf == info_flags::even)
+				return true;
 			return overall_coeff.info(inf);
 		}
 		case info_flags::algebraic: {
@@ -306,10 +305,114 @@ bool mul::info(unsigned inf) const
 			}
 			return false;
 		}
+		case info_flags::positive:
+		case info_flags::negative: {
+			if ((inf==info_flags::positive) && (flags & status_flags::is_positive))
+				return true;
+			else if ((inf==info_flags::negative) && (flags & status_flags::is_negative))
+				return true;
+			if (flags & status_flags::purely_indefinite)
+				return false;
+
+			bool pos = true;
+			epvector::const_iterator i = seq.begin(), end = seq.end();
+			while (i != end) {
+				const ex& factor = recombine_pair_to_ex(*i++);
+				if (factor.info(info_flags::positive))
+					continue;
+				else if (factor.info(info_flags::negative))
+					pos = !pos;
+				else
+					return false;
+			}
+			if (overall_coeff.info(info_flags::negative))
+				pos = !pos;
+			setflag(pos ? status_flags::is_positive : status_flags::is_negative);
+			return (inf == info_flags::positive? pos : !pos);
+		}
+		case info_flags::nonnegative: {
+			if  (flags & status_flags::is_positive)
+				return true;
+			bool pos = true;
+			epvector::const_iterator i = seq.begin(), end = seq.end();
+			while (i != end) {
+				const ex& factor = recombine_pair_to_ex(*i++);
+				if (factor.info(info_flags::nonnegative) || factor.info(info_flags::positive))
+					continue;
+				else if (factor.info(info_flags::negative))
+					pos = !pos;
+				else
+					return false;
+			}
+			return (overall_coeff.info(info_flags::negative)? pos : !pos);
+		}
+		case info_flags::posint:
+		case info_flags::negint: {
+			bool pos = true;
+			epvector::const_iterator i = seq.begin(), end = seq.end();
+			while (i != end) {
+				const ex& factor = recombine_pair_to_ex(*i++);
+				if (factor.info(info_flags::posint))
+					continue;
+				else if (factor.info(info_flags::negint))
+					pos = !pos;
+				else
+					return false;
+			}
+			if (overall_coeff.info(info_flags::negint))
+				pos = !pos;
+			else if (!overall_coeff.info(info_flags::posint))
+				return false;
+			return (inf ==info_flags::posint? pos : !pos); 
+		}
+		case info_flags::nonnegint: {
+			bool pos = true;
+			epvector::const_iterator i = seq.begin(), end = seq.end();
+			while (i != end) {
+				const ex& factor = recombine_pair_to_ex(*i++);
+				if (factor.info(info_flags::nonnegint) || factor.info(info_flags::posint))
+					continue;
+				else if (factor.info(info_flags::negint))
+					pos = !pos;
+				else
+					return false;
+			}
+			if (overall_coeff.info(info_flags::negint))
+				pos = !pos;
+			else if (!overall_coeff.info(info_flags::posint))
+				return false;
+			return pos; 
+		}
+		case info_flags::indefinite: {
+			if (flags & status_flags::purely_indefinite)
+				return true;
+			if (flags & (status_flags::is_positive | status_flags::is_negative))
+				return false;
+			epvector::const_iterator i = seq.begin(), end = seq.end();
+			while (i != end) {
+				const ex& term = recombine_pair_to_ex(*i);
+				if (term.info(info_flags::positive) || term.info(info_flags::negative))
+					return false;
+				++i;
+			}
+			setflag(status_flags::purely_indefinite);
+			return true;
+		}
 	}
 	return inherited::info(inf);
 }
 
+bool mul::is_polynomial(const ex & var) const
+{
+	for (epvector::const_iterator i=seq.begin(); i!=seq.end(); ++i) {
+		if (!i->rest.is_polynomial(var) ||
+		    (i->rest.has(var) && !i->coeff.info(info_flags::nonnegint))) {
+			return false;
+		}
+	}
+	return true;
+}
+
 int mul::degree(const ex & s) const
 {
 	// Sum up degrees of factors
@@ -400,24 +503,6 @@ ex mul::eval(int level) const
 		           setflag(status_flags::dynallocated);
 	}
 	
-#ifdef DO_GINAC_ASSERT
-	epvector::const_iterator i = seq.begin(), end = seq.end();
-	while (i != end) {
-		GINAC_ASSERT((!is_exactly_a<mul>(i->rest)) ||
-		             (!(ex_to<numeric>(i->coeff).is_integer())));
-		GINAC_ASSERT(!(i->is_canonical_numeric()));
-		if (is_exactly_a<numeric>(recombine_pair_to_ex(*i)))
-		    print(print_tree(std::cerr));
-		GINAC_ASSERT(!is_exactly_a<numeric>(recombine_pair_to_ex(*i)));
-		/* for paranoia */
-		expair p = split_ex_to_pair(recombine_pair_to_ex(*i));
-		GINAC_ASSERT(p.rest.is_equal(i->rest));
-		GINAC_ASSERT(p.coeff.is_equal(i->coeff));
-		/* end paranoia */
-		++i;
-	}
-#endif // def DO_GINAC_ASSERT
-	
 	if (flags & status_flags::evaluated) {
 		GINAC_ASSERT(seq.size()>0);
 		GINAC_ASSERT(seq.size()>1 || !overall_coeff.is_equal(_ex1));
@@ -452,7 +537,7 @@ ex mul::eval(int level) const
 		       )->setflag(status_flags::dynallocated | status_flags::evaluated);
 	} else if ((seq_size >= 2) && (! (flags & status_flags::expanded))) {
 		// Strip the content and the unit part from each term. Thus
-		// things like (-x+a)*(3*x-3*a) automagically turn into - 3*(x-a)2
+		// things like (-x+a)*(3*x-3*a) automagically turn into - 3*(x-a)^2
 
 		epvector::const_iterator last = seq.end();
 		epvector::const_iterator i = seq.begin();
@@ -504,8 +589,7 @@ ex mul::eval(int level) const
 			primitive->setflag(status_flags::dynallocated);
 			primitive->clearflag(status_flags::hash_calculated);
 			primitive->overall_coeff = ex_to<numeric>(primitive->overall_coeff).div_dyn(c);
-			for (epvector::iterator ai = primitive->seq.begin();
-					ai != primitive->seq.end(); ++ai)
+			for (epvector::iterator ai = primitive->seq.begin(); ai != primitive->seq.end(); ++ai)
 				ai->coeff = ex_to<numeric>(ai->coeff).div_dyn(c);
 			
 			s->push_back(expair(*primitive, _ex1));
@@ -637,7 +721,7 @@ ex mul::eval_ncmul(const exvector & v) const
 	return inherited::eval_ncmul(v);
 }
 
-bool tryfactsubs(const ex & origfactor, const ex & patternfactor, int & nummatches, lst & repls)
+bool tryfactsubs(const ex & origfactor, const ex & patternfactor, int & nummatches, exmap& repls)
 {	
 	ex origbase;
 	int origexponent;
@@ -669,7 +753,7 @@ bool tryfactsubs(const ex & origfactor, const ex & patternfactor, int & nummatch
 		patternexpsign = 1;
 	}
 
-	lst saverepls = repls;
+	exmap saverepls = repls;
 	if (origexponent < patternexponent || origexpsign != patternexpsign || !origbase.match(patternbase,saverepls))
 		return false;
 	repls = saverepls;
@@ -688,17 +772,20 @@ bool tryfactsubs(const ex & origfactor, const ex & patternfactor, int & nummatch
   * that already have been replaced by previous substitutions and matched[i]
   * is true for factors that have been matched by the current match.
   */
-bool algebraic_match_mul_with_mul(const mul &e, const ex &pat, lst &repls,
-		int factor, int &nummatches, const std::vector<bool> &subsed,
-		std::vector<bool> &matched)
+bool algebraic_match_mul_with_mul(const mul &e, const ex &pat, exmap& repls,
+                                  int factor, int &nummatches, const std::vector<bool> &subsed,
+                                  std::vector<bool> &matched)
 {
-	if (factor == pat.nops())
+	GINAC_ASSERT(subsed.size() == e.nops());
+	GINAC_ASSERT(matched.size() == e.nops());
+
+	if (factor == (int)pat.nops())
 		return true;
 
 	for (size_t i=0; i<e.nops(); ++i) {
 		if(subsed[i] || matched[i])
 			continue;
-		lst newrepls = repls;
+		exmap newrepls = repls;
 		int newnummatches = nummatches;
 		if (tryfactsubs(e.op(i), pat.op(factor), newnummatches, newrepls)) {
 			matched[i] = true;
@@ -721,10 +808,10 @@ bool mul::has(const ex & pattern, unsigned options) const
 	if(!(options&has_options::algebraic))
 		return basic::has(pattern,options);
 	if(is_a<mul>(pattern)) {
-		lst repls;
+		exmap repls;
 		int nummatches = std::numeric_limits<int>::max();
-		std::vector<bool> subsed(seq.size(), false);
-		std::vector<bool> matched(seq.size(), false);
+		std::vector<bool> subsed(nops(), false);
+		std::vector<bool> matched(nops(), false);
 		if(algebraic_match_mul_with_mul(*this, pattern, repls, 0, nummatches,
 				subsed, matched))
 			return true;
@@ -734,8 +821,7 @@ bool mul::has(const ex & pattern, unsigned options) const
 
 ex mul::algebraic_subs_mul(const exmap & m, unsigned options) const
 {	
-	std::vector<bool> subsed(seq.size(), false);
-	exvector subsresult(seq.size());
+	std::vector<bool> subsed(nops(), false);
 	ex divide_by = 1;
 	ex multiply_by = 1;
 
@@ -744,8 +830,8 @@ ex mul::algebraic_subs_mul(const exmap & m, unsigned options) const
 		if (is_exactly_a<mul>(it->first)) {
 retry1:
 			int nummatches = std::numeric_limits<int>::max();
-			std::vector<bool> currsubsed(seq.size(), false);
-			lst repls;
+			std::vector<bool> currsubsed(nops(), false);
+			exmap repls;
 			
 			if(!algebraic_match_mul_with_mul(*this, it->first, repls, 0, nummatches, subsed, currsubsed))
 				continue;
@@ -754,10 +840,10 @@ retry1:
 				if (currsubsed[j])
 					subsed[j] = true;
 			ex subsed_pattern
-				= it->first.subs(ex(repls), subs_options::no_pattern);
+				= it->first.subs(repls, subs_options::no_pattern);
 			divide_by *= power(subsed_pattern, nummatches);
 			ex subsed_result
-				= it->second.subs(ex(repls), subs_options::no_pattern);
+				= it->second.subs(repls, subs_options::no_pattern);
 			multiply_by *= power(subsed_result, nummatches);
 			goto retry1;
 
@@ -765,14 +851,14 @@ retry1:
 
 			for (size_t j=0; j<this->nops(); j++) {
 				int nummatches = std::numeric_limits<int>::max();
-				lst repls;
+				exmap repls;
 				if (!subsed[j] && tryfactsubs(op(j), it->first, nummatches, repls)){
 					subsed[j] = true;
 					ex subsed_pattern
-						= it->first.subs(ex(repls), subs_options::no_pattern);
+						= it->first.subs(repls, subs_options::no_pattern);
 					divide_by *= power(subsed_pattern, nummatches);
 					ex subsed_result
-						= it->second.subs(ex(repls), subs_options::no_pattern);
+						= it->second.subs(repls, subs_options::no_pattern);
 					multiply_by *= power(subsed_result, nummatches);
 				}
 			}
@@ -792,6 +878,38 @@ retry1:
 	return ((*this)/divide_by)*multiply_by;
 }
 
+ex mul::conjugate() const
+{
+	// The base class' method is wrong here because we have to be careful at
+	// branch cuts. power::conjugate takes care of that already, so use it.
+	epvector *newepv = 0;
+	for (epvector::const_iterator i=seq.begin(); i!=seq.end(); ++i) {
+		if (newepv) {
+			newepv->push_back(split_ex_to_pair(recombine_pair_to_ex(*i).conjugate()));
+			continue;
+		}
+		ex x = recombine_pair_to_ex(*i);
+		ex c = x.conjugate();
+		if (c.is_equal(x)) {
+			continue;
+		}
+		newepv = new epvector;
+		newepv->reserve(seq.size());
+		for (epvector::const_iterator j=seq.begin(); j!=i; ++j) {
+			newepv->push_back(*j);
+		}
+		newepv->push_back(split_ex_to_pair(c));
+	}
+	ex x = overall_coeff.conjugate();
+	if (!newepv && are_ex_trivially_equal(x, overall_coeff)) {
+		return *this;
+	}
+	ex result = thisexpairseq(newepv ? *newepv : seq, x);
+	delete newepv;
+	return result;
+}
+
+
 // protected
 
 /** Implementation of ex::diff() for a product.  It applies the product rule.
@@ -854,11 +972,11 @@ unsigned mul::return_type() const
 	// all factors checked
 	return all_commutative ? return_types::commutative : return_types::noncommutative;
 }
-   
-tinfo_t mul::return_type_tinfo() const
+
+return_type_t mul::return_type_tinfo() const
 {
 	if (seq.empty())
-		return this;  // mul without factors: should not happen
+		return make_return_type_t<mul>(); // mul without factors: should not happen
 	
 	// return type_info of first noncommutative element
 	epvector::const_iterator i = seq.begin(), end = seq.end();
@@ -868,7 +986,7 @@ tinfo_t mul::return_type_tinfo() const
 		++i;
 	}
 	// no noncommutative element found, should not happen
-	return this;
+	return make_return_type_t<mul>();
 }
 
 ex mul::thisexpairseq(const epvector & v, const ex & oc, bool do_index_renaming) const
@@ -890,7 +1008,7 @@ expair mul::split_ex_to_pair(const ex & e) const
 	}
 	return expair(e,_ex1);
 }
-	
+
 expair mul::combine_ex_with_coeff_to_pair(const ex & e,
                                           const ex & c) const
 {
@@ -903,7 +1021,7 @@ expair mul::combine_ex_with_coeff_to_pair(const ex & e,
 
 	return split_ex_to_pair(power(e,c));
 }
-	
+
 expair mul::combine_pair_with_coeff_to_pair(const expair & p,
                                             const ex & c) const
 {
@@ -916,7 +1034,7 @@ expair mul::combine_pair_with_coeff_to_pair(const expair & p,
 
 	return split_ex_to_pair(power(recombine_pair_to_ex(p),c));
 }
-	
+
 ex mul::recombine_pair_to_ex(const expair & p) const
 {
 	if (ex_to<numeric>(p.coeff).is_equal(*_num1_p)) 
@@ -928,22 +1046,22 @@ ex mul::recombine_pair_to_ex(const expair & p) const
 bool mul::expair_needs_further_processing(epp it)
 {
 	if (is_exactly_a<mul>(it->rest) &&
-		ex_to<numeric>(it->coeff).is_integer()) {
+	    ex_to<numeric>(it->coeff).is_integer()) {
 		// combined pair is product with integer power -> expand it
 		*it = split_ex_to_pair(recombine_pair_to_ex(*it));
 		return true;
 	}
 	if (is_exactly_a<numeric>(it->rest)) {
+		if (it->coeff.is_equal(_ex1)) {
+			// pair has coeff 1 and must be moved to the end
+			return true;
+		}
 		expair ep = split_ex_to_pair(recombine_pair_to_ex(*it));
 		if (!ep.is_equal(*it)) {
 			// combined pair is a numeric power which can be simplified
 			*it = ep;
 			return true;
 		}
-		if (it->coeff.is_equal(_ex1)) {
-			// combined pair has coeff 1 and must be moved to the end
-			return true;
-		}
 	}
 	return false;
 }       
@@ -1217,4 +1335,6 @@ std::auto_ptr<epvector> mul::expandchildren(unsigned options) const
 	return std::auto_ptr<epvector>(0); // nothing has changed
 }
 
+GINAC_BIND_UNARCHIVER(mul);
+
 } // namespace GiNaC
