@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's lexer. */
 
 /*
- *  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2014 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 #include <iostream>
 #include <sstream>
 #include <string>
+#include <cstdio>
 
 namespace GiNaC {
 
@@ -42,12 +43,12 @@ int lexer::gettok()
 	// Skip any whitespace.
 	c = skipspace(input, c, line_num);
 
-	// identifier: [a-zA-Z][a-zA-Z0-9]*
+	// identifier: [a-zA-Z][a-zA-Z0-9_]*
 	if (isalpha(c)) { 
 		str = c;
 		do {
 			c = input->get();
-			if (isalnum(c))
+			if ( isalnum(c) || c=='_' )
 				str += c;
 			else
 				break;
