@@ -3,7 +3,7 @@
  *  Implementation of the parser context. */
 
 /*
- *  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2014 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,19 +27,12 @@
 
 namespace GiNaC {
 
-symbol
+ex
 find_or_insert_symbol(const std::string& name, symtab& syms, const bool strict)
 {
 	symtab::const_iterator p = syms.find(name);
-	if (p != syms.end()) {
-		if (is_a<symbol>(p->second))
-			return ex_to<symbol>(p->second);
-		else
-			throw std::invalid_argument(
-				std::string("find_or_insert_symbol: name \"")
-				+ name + "\" does not correspond to a symbol");
-	}
-
+	if (p != syms.end())
+		return p->second;
 
 	if (strict)
 		throw std::invalid_argument(
