@@ -4,7 +4,7 @@
  *  internal header file. */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2014 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,8 +21,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef __GINAC_TOSTRING_H__
-#define __GINAC_TOSTRING_H__
+#ifndef GINAC_TOSTRING_H
+#define GINAC_TOSTRING_H
 
 #include <sstream>
 
@@ -38,4 +38,4 @@ std::string ToString(const T & t)
 
 } // namespace GiNaC
 
-#endif // ndef __GINAC_TOSTRING_H__
+#endif // ndef GINAC_TOSTRING_H
