@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's wildcard objects. */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2014 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,11 +20,12 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <iostream>
-
 #include "wildcard.h"
 #include "archive.h"
 #include "utils.h"
+#include "hash_seed.h"
+
+#include <iostream>
 
 namespace GiNaC {
 
@@ -37,7 +38,7 @@ GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(wildcard, basic,
 // default constructor
 //////////
 
-wildcard::wildcard() : inherited(TINFO_wildcard), label(0)
+wildcard::wildcard() : label(0)
 {
 	setflag(status_flags::evaluated | status_flags::expanded);
 }
@@ -46,7 +47,7 @@ wildcard::wildcard() : inherited(TINFO_wildcard), label(0)
 // other constructors
 //////////
 
-wildcard::wildcard(unsigned l) : inherited(TINFO_wildcard), label(l)
+wildcard::wildcard(unsigned l) : label(l)
 {
 	setflag(status_flags::evaluated | status_flags::expanded);
 }
@@ -55,11 +56,13 @@ wildcard::wildcard(unsigned l) : inherited(TINFO_wildcard), label(l)
 // archiving
 //////////
 
-wildcard::wildcard(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst)
+void wildcard::read_archive(const archive_node& n, lst& sym_lst)
 {
+	inherited::read_archive(n, sym_lst);
 	n.find_unsigned("label", label);
 	setflag(status_flags::evaluated | status_flags::expanded);
 }
+GINAC_BIND_UNARCHIVER(wildcard);
 
 void wildcard::archive(archive_node &n) const
 {
@@ -67,8 +70,6 @@ void wildcard::archive(archive_node &n) const
 	n.add_unsigned("label", label);
 }
 
-DEFAULT_UNARCHIVE(wildcard)
-
 //////////
 // functions overriding virtual functions from base classes
 //////////
@@ -104,14 +105,15 @@ void wildcard::do_print_python_repr(const print_python_repr & c, unsigned level)
 unsigned wildcard::calchash() const
 {
 	// this is where the schoolbook method
-	// (golden_ratio_hash(tinfo()) ^ label)
+	// (golden_ratio_hash(typeid(*this).name()) ^ label)
 	// is not good enough yet...
-	hashvalue = golden_ratio_hash(golden_ratio_hash(tinfo()) ^ label);
+	unsigned seed = make_hash_seed(typeid(*this));
+	hashvalue = golden_ratio_hash(seed ^ label);
 	setflag(status_flags::hash_calculated);
 	return hashvalue;
 }
 
-bool wildcard::match(const ex & pattern, lst & repl_lst) const
+bool wildcard::match(const ex & pattern, exmap& repl_lst) const
 {
 	// Wildcards must match each other exactly (this is required for
 	// subs() to work properly because in the final step it substitutes
